//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_DiskImages.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_diskimages {
//---------------------------------------------------------------------------
bool __fastcall TDiskImageSpec::operator ==(const TDiskImageSpec& AOther)
{
	if(this == &AOther) {
		return true;
	}
	return
		DiskType == AOther.DiskType &&
		C == AOther.C &&
		H == AOther.H &&
		R == AOther.R &&
		N == AOther.N;
}
//---------------------------------------------------------------------------
bool __fastcall TDiskImageSpec::operator !=(const TDiskImageSpec& AOther)
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TDiskImage::CreateBlankDiskImage(int ADiskType, int AC, int AH, int AR, int AN, TStream* ABuffer)
{
	int LSectorSize = SectorHeaderSize + DetermineSectorDataSize(AN);
	int LTrackSize = LSectorSize * AR;
	int LTrackCount = AC * AH;
	int LDiskSize = HeaderSize + LTrackSize * LTrackCount;
	TAutoArray<byte> LDisk = new byte[LDiskSize];
	::ZeroMemory(LDisk, LDiskSize);
	WriteInteger(LDisk, WriteProtectedOffset, WriteProtectedSize, WriteProtected_No);
	WriteInteger(LDisk, DiskTypeOffset, DiskTypeSize, ADiskType);
	WriteInteger(LDisk, SizeOffset, SizeSize, LDiskSize);
	for(int LI = 0; LI < LTrackCount; ++LI) {
		WriteInteger(LDisk, TrackTableOffset + TrackTableEntrySize * LI, TrackTableEntrySize, TracksOffset + LTrackSize * LI);
	}
	int LSectorOffset = TracksOffset - LSectorSize;
	for(int LC = 0; LC < AC; ++LC) {
		for(int LH = 0; LH < AH; ++LH) {
			for(int LR = 0; LR < AR; ++LR) {
				LSectorOffset += LSectorSize;
				WriteInteger(LDisk, LSectorOffset + SectorCOffset, SectorCSize, LC);
				WriteInteger(LDisk, LSectorOffset + SectorHOffset, SectorHSize, LH);
				WriteInteger(LDisk, LSectorOffset + SectorROffset, SectorRSize, LR + 1);
				WriteInteger(LDisk, LSectorOffset + SectorNOffset, SectorNSize, AN);
				WriteInteger(LDisk, LSectorOffset + SectorCountOffset, SectorCountSize, AR);
				WriteInteger(LDisk, LSectorOffset + SectorDensityOffset, SectorDensitySize, DetermineSectorDensity(ADiskType));
				WriteInteger(LDisk, LSectorOffset + SectorDeletedOffset, SectorDeletedSize, SectorDeleted_No);
				WriteInteger(LDisk, LSectorOffset + SectorStatusOffset, SectorStatusSize, SectorStatus_Normal);
				WriteInteger(LDisk, LSectorOffset + SectorDataSizeOffset, SectorDataSizeSize, LSectorSize - SectorHeaderSize);
			}
		}
	}
	ABuffer->Size = 0;
	ABuffer->WriteBuffer(LDisk, LDiskSize);
}
//---------------------------------------------------------------------------
int __fastcall TDiskImage::DetermineSectorDataSize(int AN)
{
	return 128 << AN; // 128 * Power(2, AN)
}
//---------------------------------------------------------------------------
int __fastcall TDiskImage::DetermineSectorDensity(int ADiskType)
{
	switch(ADiskType) {
	case DiskType_2d:
	case DiskType_2dd:
	case DiskType_2hd:
		return SectorDensity_Double;
	default:
		throw EDiskImageError("The program does not support the disk type.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TDiskImage::ReadInteger(const void* AMemory, int AOffset, int ASize)
{
	int LValue = 0;
	for(int LI = 0; LI < ASize; ++LI) {
		LValue |= static_cast<int>(*(static_cast<const byte*>(AMemory) + AOffset + LI)) << (8 * LI);
	}
	return LValue;
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::WriteInteger(void* AMemory, int AOffset, int ASize, int AValue)
{
	for(int LI = 0; LI < ASize; ++LI) {
		*(static_cast<byte*>(AMemory) + AOffset + LI) = (static_cast<unsigned int>(AValue) >> (8 * LI)) & 0xFF;
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDiskImage::ReadString(const void* AMemory, int AOffset, int ASize)
{
	return AnsiString(reinterpret_cast<const char*>(static_cast<const byte*>(AMemory) + AOffset));
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::WriteString(void* AMemory, int AOffset, int ASize, const AnsiString& AValue)
{
	if(AValue.Length() >= ASize) {
		throw EDiskImageError("The program cannot write the string data to a disk image.");
	}
	::FillMemory(static_cast<byte*>(AMemory) + AOffset, ASize, '\0');
	::CopyMemory(static_cast<byte*>(AMemory) + AOffset, AValue.c_str(), AValue.Length());
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::Initialize(int ADiskType, int AC, int AH, int AR, int AN)
{
	try {
		FBuffer = new TMemoryStream();
		CreateBlankDiskImage(ADiskType, AC, AH, AR, AN, Buffer);
		UpdateCache();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::Uninitialize()
{
	delete FBuffer;
	FBuffer = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::UpdateCache()
{
	FName = ReadString(Buffer->Memory, NameOffset, NameSize);
	FWriteProtected = ReadInteger(Buffer->Memory, WriteProtectedOffset, WriteProtectedSize);
	FDiskType = ReadInteger(Buffer->Memory, DiskTypeOffset, DiskTypeSize);
	FSize = ReadInteger(Buffer->Memory, SizeOffset, SizeSize);
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::UpdateBuffer()
{
	WriteString(Buffer->Memory, NameOffset, NameSize, Name);
	WriteInteger(Buffer->Memory, WriteProtectedOffset, WriteProtectedSize, WriteProtected);
	WriteInteger(Buffer->Memory, DiskTypeOffset, DiskTypeSize, DiskType);
	WriteInteger(Buffer->Memory, SizeOffset, SizeSize, Size);
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::SetName(const AnsiString& AName)
{
	if(AName.Length() > MaxNameLength) {
		throw EDiskImageError("The program cannot set the name property of a disk image.");
	}
	if(AName == Name) {
		return;
	}
	FName = AName;
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::SetWriteProtected(int AWriteProtected)
{
	switch(AWriteProtected) {
	case WriteProtected_Yes:
	case WriteProtected_No:
		break;
	default:
		throw EDiskImageError("The program cannot set the write-protected property of a disk image.");
	}
	if(AWriteProtected == WriteProtected) {
		return;
	}
	FWriteProtected = AWriteProtected;
}
//---------------------------------------------------------------------------
__fastcall TDiskImage::TDiskImage(const TDiskImageSpec& ASpec) :
	TObject(),
	FName(),
	FWriteProtected(0),
	FDiskType(0),
	FBuffer(NULL),
	FSize(0)
{
	Initialize(ASpec.DiskType, ASpec.C, ASpec.H, ASpec.R, ASpec.N);
}
//---------------------------------------------------------------------------
__fastcall TDiskImage::TDiskImage(int ADiskType, int AC, int AH, int AR, int AN) :
	TObject(),
	FName(),
	FWriteProtected(0),
	FDiskType(0),
	FBuffer(NULL),
	FSize(0)
{
	Initialize(ADiskType, AC, AH, AR, AN);
}
//---------------------------------------------------------------------------
__fastcall TDiskImage::TDiskImage() :
	TObject(),
	FName(),
	FWriteProtected(0),
	FDiskType(0),
	FBuffer(NULL),
	FSize(0)
{
	Initialize(DiskType_2d, 0, 0, 0, 0);
}
//---------------------------------------------------------------------------
__fastcall TDiskImage::~TDiskImage()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::Assign(TDiskImage* ASource)
{
	if(ASource == this) {
		return;
	}
	ASource->UpdateBuffer();
	Buffer->Size = 0;
	ASource->Buffer->SaveToStream(Buffer);
	UpdateCache();
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::LoadFromStream(TStream* AStream)
{
	try {
		TAutoPointer<TDiskImage> LDiskImage = new TDiskImage();
		LDiskImage->Buffer->Size = HeaderSize;
		LDiskImage->Buffer->Position = 0;
		AStream->ReadBuffer(LDiskImage->Buffer->Memory, HeaderSize);
		LDiskImage->Buffer->Size = ReadInteger(LDiskImage->Buffer->Memory, SizeOffset, SizeSize);
		LDiskImage->Buffer->Position = HeaderSize;
		AStream->ReadBuffer(static_cast<byte*>(LDiskImage->Buffer->Memory) + HeaderSize, LDiskImage->Buffer->Size - HeaderSize);
		LDiskImage->UpdateCache();
		Assign(LDiskImage);
	}
	catch(EReadError& LException) {
		throw EDiskImageError("The program could not load the disk image.");
	}
	catch(EStreamError& LException) {
		throw EInvalidDiskImage("The data is not a valid disk image.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDiskImage::SaveToStream(TStream* AStream)
{
	try {
		UpdateBuffer();
		AStream->WriteBuffer(Buffer->Memory, Buffer->Size);
	}
	catch(EWriteError& LException) {
		throw EDiskImageError("The program could not save the disk image.");
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TDiskImageList::TDiskImageList() :
	TObjectList()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDiskImageList::TDiskImageList(bool AOwnsObjects) :
	TObjectList(AOwnsObjects)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDiskImageList::~TDiskImageList()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDiskImageList::LoadFromFile(const AnsiString& APath)
{
	try {
		TAutoPointer<TMemoryStream> LStream = new TMemoryStream();
		LStream->LoadFromFile(APath);
		LoadFromStream(LStream);
	}
	catch(EStreamError& LException) {
		LException.Message = "The program could not load the disk images.";
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDiskImageList::SaveToFile(const AnsiString& APath)
{
	try {
		TAutoPointer<TMemoryStream> LStream = new TMemoryStream();
		SaveToStream(LStream);
		LStream->SaveToFile(APath);
	}
	catch(EStreamError& LException) {
		LException.Message = "The program could not save the disk images.";
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDiskImageList::LoadFromStream(TStream* AStream)
{
	TAutoPointer<TObjectList> LList = new TObjectList();
	try {
		__int64 LStreamSize = AStream->Size;
		AStream->Position = 0;
		while(AStream->Position < LStreamSize) {
			TAutoPointer<TDiskImage> LItem = new TDiskImage();
			LItem->LoadFromStream(AStream);
			LList->Add(LItem);
			LItem.Release();
		}
	}
	catch(EStreamError& LException) {
		LException.Message = "The program could not load the disk images.";
		throw;
	}
	Assign(LList);
	LList->OwnsObjects = false;
}
//---------------------------------------------------------------------------
void __fastcall TDiskImageList::SaveToStream(TStream* AStream)
{
	try {
		AStream->Size = 0;
		for(int LI = 0; LI < Count; ++LI) {
			static_cast<TDiskImage*>(Items[LI])->SaveToStream(AStream);
		}
	}
	catch(EStreamError& LException) {
		LException.Message = "The program could not save the disk images.";
		throw;
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TDiskImageUtilities::EvaluateWriteProtected(int AWriteProtected)
{
	switch(AWriteProtected) {
	case TDiskImage::WriteProtected_Yes:
		return 0;
	case TDiskImage::WriteProtected_No:
		return 1;
	default:
		throw EDiskImageError("The program does not support the write-protected mode.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TDiskImageUtilities::EvaluateDiskType(int ADiskType)
{
	switch(ADiskType) {
	case TDiskImage::DiskType_2d:
		return 0;
	case TDiskImage::DiskType_2dd:
		return 1;
	case TDiskImage::DiskType_2hd:
		return 2;
	default:
		throw EDiskImageError("The program does not support the disk type.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDiskImageUtilities::DescribeName(const AnsiString& AName)
{
	return AName;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDiskImageUtilities::DescribeWriteProtected(int AWriteProtected)
{
	switch(AWriteProtected) {
	case TDiskImage::WriteProtected_Yes:
		return "Yes";
	case TDiskImage::WriteProtected_No:
		return "No";
	default:
		throw EDiskImageError("The program does not support the write-protected mode.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDiskImageUtilities::DescribeDiskType(int ADiskType)
{
	switch(ADiskType) {
	case TDiskImage::DiskType_2d:
		return "2D";
	case TDiskImage::DiskType_2dd:
		return "2DD";
	case TDiskImage::DiskType_2hd:
		return "2HD";
	default:
		throw EDiskImageError("The program does not support the disk type.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDiskImageUtilities::DescribeSize(int ASize)
{
	return DescribeSize(static_cast<__int64>(ASize));
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDiskImageUtilities::DescribeSize(__int64 ASize)
{
	AnsiString LString = ASize < 0 ? -ASize : ASize;
	for(int LI = LString.Length() - 3 + 1; LI > 1; LI -= 3) {
		LString.Insert(",", LI);
	}
	if(ASize < 0) {
		LString = "-" + LString;
	}
	return LString + " bytes";
}
//---------------------------------------------------------------------------
int __fastcall TDiskImageUtilities::CompareName(const AnsiString& AName1, const AnsiString& AName2)
{
	return AnsiCompareStr(AName1, AName2);
}
//---------------------------------------------------------------------------
int __fastcall TDiskImageUtilities::CompareWriteProtected(int AWriteProtected1, int AWriteProtected2)
{
	return
		EvaluateWriteProtected(AWriteProtected1) -
		EvaluateWriteProtected(AWriteProtected2);
}
//---------------------------------------------------------------------------
int __fastcall TDiskImageUtilities::CompareDiskType(int ADiskType1, int ADiskType2)
{
	return
		EvaluateWriteProtected(ADiskType1) -
		EvaluateWriteProtected(ADiskType2);
}
//---------------------------------------------------------------------------
int __fastcall TDiskImageUtilities::CompareSize(int ASize1, int ASize2)
{
	return ASize1 - ASize2;
}
//---------------------------------------------------------------------------
} // End namespace App_diskimages
//---------------------------------------------------------------------------

