//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_MainForm.h"
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TAppMainForm* AppMainForm = NULL;
//---------------------------------------------------------------------------
const AnsiString TAppMainForm::ProgramTitle = "D88 Edit";
const AnsiString TAppMainForm::FormCaptionFormat = "%s - %s";
const AnsiString TAppMainForm::UndoActionCaptionFormat = "&Undo - %s";
const AnsiString TAppMainForm::UndoActionHintFormat = "Undo - %s";
const AnsiString TAppMainForm::RedoActionCaptionFormat = "&Redo - %s";
const AnsiString TAppMainForm::RedoActionHintFormat = "Redo - %s";
const AnsiString TAppMainForm::ItemCountStatusFormat = "%d items";
const AnsiString TAppMainForm::SelectionStatusFormat = "%d items are selected";
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ActionList1Execute(TBasicAction* AAction, bool& AHandled)
{
	AHandled = false;
	if(AAction->OnExecute == NULL) {
		return;
	}
	TCursor LCursor = Screen->Cursor;
	Screen->Cursor = crHourGlass;
	try {
		AHandled = true;
		AAction->OnExecute(AAction);
	}
	catch(...) {
		Screen->Cursor = LCursor;
		throw;
	}
	Screen->Cursor = LCursor;
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ActionList1Update(TBasicAction* AAction, bool& AHandled)
{
	AHandled = true;
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileNewActionExecute(TObject* ASender)
{
	FileHandler->New();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileNewActionUpdate(TObject* ASender)
{
	FileNewAction->Enabled = FileHandler->CanNew();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileOpenActionExecute(TObject* ASender)
{
	FileHandler->Open();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileOpenActionUpdate(TObject* ASender)
{
	FileOpenAction->Enabled = FileHandler->CanOpen();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveActionExecute(TObject* ASender)
{
	FileHandler->Save();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveActionUpdate(TObject* ASender)
{
	FileSaveAction->Enabled = FileHandler->CanSave();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveAsActionExecute(TObject* ASender)
{
	FileHandler->SaveAs();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveAsActionUpdate(TObject* ASender)
{
	FileSaveAsAction->Enabled = FileHandler->CanSaveAs();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileBackUpActionExecute(TObject* ASender)
{
	FileHandler->BackUp();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileBackUpActionUpdate(TObject* ASender)
{
	FileBackUpAction->Enabled = FileHandler->CanBackUp();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileBackUpOnSaveActionExecute(TObject* ASender)
{
	FileHandler->ToggleBackUpOnSave();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileBackUpOnSaveActionUpdate(TObject* ASender)
{
	FileBackUpOnSaveAction->Checked = FileHandler->BackUpOnSave;
	FileBackUpOnSaveAction->Enabled = FileHandler->CanToggleBackUpOnSave();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileAddNewActionExecute(TObject* ASender)
{
	FileHandler->AddNew();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileAddNewActionUpdate(TObject* ASender)
{
	FileAddNewAction->Enabled = FileHandler->CanAddNew();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileAddExistentActionExecute(TObject* ASender)
{
	FileHandler->AddExistent();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileAddExistentActionUpdate(TObject* ASender)
{
	FileAddExistentAction->Enabled = FileHandler->CanAddExistent();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveToSingleFileActionExecute(TObject* ASender)
{
	FileHandler->SaveToSingleFile();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveToSingleFileActionUpdate(TObject* ASender)
{
	FileSaveToSingleFileAction->Enabled = FileHandler->CanSaveToSingleFile();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveToMultipleFilesActionExecute(TObject* ASender)
{
	FileHandler->SaveToMultipleFiles();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileSaveToMultipleFilesActionUpdate(TObject* ASender)
{
	FileSaveToMultipleFilesAction->Enabled = FileHandler->CanSaveToMultipleFiles();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileExitActionExecute(TObject* ASender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileExitActionUpdate(TObject* ASender)
{
	FileExitAction->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileDropFilesActionExecute(TObject* ASender)
{
	try {
		FileHandler->DropFiles(FileDropFilesAction->Files);
	}
	catch(...) {
		FileDropFilesAction->Files->Clear();
		throw;
	}
	FileDropFilesAction->Files->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileDropFilesActionUpdate(TObject* ASender)
{
	::DragAcceptFiles(Handle, FileHandler->CanDropFiles());
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditUndoActionExecute(TObject* ASender)
{
	FileHandler->Undo();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditUndoActionUpdate(TObject* ASender)
{
	EditUndoAction->Caption = AnsiString().sprintf(
		UndoActionCaptionFormat.c_str(),
		StringToActionCaption(FileHandler->UndoDescription).c_str());
	EditUndoAction->Hint = AnsiString().sprintf(
		UndoActionHintFormat.c_str(),
		FileHandler->UndoDescription.c_str());
	EditUndoAction->Enabled = FileHandler->CanUndo();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditRedoActionExecute(TObject* ASender)
{
	FileHandler->Redo();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditRedoActionUpdate(TObject* ASender)
{
	EditRedoAction->Caption = AnsiString().sprintf(
		RedoActionCaptionFormat.c_str(),
		StringToActionCaption(FileHandler->RedoDescription).c_str());
	EditRedoAction->Hint = AnsiString().sprintf(
		RedoActionHintFormat.c_str(),
		FileHandler->RedoDescription.c_str());
	EditRedoAction->Enabled = FileHandler->CanRedo();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditDeleteActionExecute(TObject* ASender)
{
	FileHandler->Delete();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditDeleteActionUpdate(TObject* ASender)
{
	EditDeleteAction->Enabled = FileHandler->CanDelete();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditSelectAllActionExecute(TObject* ASender)
{
	FileHandler->SelectAll();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditSelectAllActionUpdate(TObject* ASender)
{
	EditSelectAllAction->Enabled = FileHandler->CanSelectAll();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditReverseSelectionActionExecute(TObject* ASender)
{
	FileHandler->ReverseSelection();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditReverseSelectionActionUpdate(TObject* ASender)
{
	EditReverseSelectionAction->Enabled = FileHandler->CanReverseSelection();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditDuplicateActionExecute(TObject* ASender)
{
	FileHandler->Duplicate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::EditDuplicateActionUpdate(TObject* ASender)
{
	EditDuplicateAction->Enabled = FileHandler->CanDuplicate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesNameCopyActionExecute(TObject* ASender)
{
	FileHandler->CopyNameProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesNameCopyActionUpdate(TObject* ASender)
{
	PropertiesNameCopyAction->Enabled = FileHandler->CanCopyNameProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesNamePasteActionExecute(TObject* ASender)
{
	FileHandler->PasteNameProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesNamePasteActionUpdate(TObject* ASender)
{
	PropertiesNamePasteAction->Enabled = FileHandler->CanPasteNameProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesNameEnterActionExecute(TObject* ASender)
{
	FileHandler->EnterNameProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesNameEnterActionUpdate(TObject* ASender)
{
	PropertiesNameEnterAction->Enabled = FileHandler->CanEnterNameProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesWriteProtectedSetToYesActionExecute(TObject* ASender)
{
	FileHandler->SetWriteProtectedPropertyToYes();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesWriteProtectedSetToYesActionUpdate(TObject* ASender)
{
	PropertiesWriteProtectedSetToYesAction->Enabled = FileHandler->CanSetWriteProtectedPropertyToYes();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesWriteProtectedSetToNoActionExecute(TObject* ASender)
{
	FileHandler->SetWriteProtectedPropertyToNo();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesWriteProtectedSetToNoActionUpdate(TObject* ASender)
{
	PropertiesWriteProtectedSetToNoAction->Enabled = FileHandler->CanSetWriteProtectedPropertyToNo();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesWriteProtectedToggleActionExecute(TObject* ASender)
{
	FileHandler->ToggleWriteProtectedProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesWriteProtectedToggleActionUpdate(TObject* ASender)
{
	PropertiesWriteProtectedToggleAction->Enabled = FileHandler->CanToggleWriteProtectedProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesSizeSumActionExecute(TObject* ASender)
{
	FileHandler->SumSizeProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::PropertiesSizeSumActionUpdate(TObject* ASender)
{
	PropertiesSizeSumAction->Enabled = FileHandler->CanSumSizeProperty();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewToolbarActionExecute(TObject* ASender)
{
	ViewHandler->ToggleShowToolbar();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewToolbarActionUpdate(TObject* ASender)
{
	ViewToolbarAction->Enabled = ViewHandler->CanToggleShowToolbar();
	ViewToolbarAction->Checked = ViewHandler->ShowToolbar;
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewStatusBarActionExecute(TObject* ASender)
{
	ViewHandler->ToggleShowStatusBar();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewStatusBarActionUpdate(TObject* ASender)
{
	ViewStatusBarAction->Enabled = ViewHandler->CanToggleShowStatusBar();
	ViewStatusBarAction->Checked = ViewHandler->ShowStatusBar;
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewRowSelectActionExecute(TObject* ASender)
{
	ViewHandler->ToggleRowSelect();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewRowSelectActionUpdate(TObject* ASender)
{
	ViewRowSelectAction->Enabled = ViewHandler->CanToggleRowSelect();
	ViewRowSelectAction->Checked = ViewHandler->RowSelect;
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortByNameActionExecute(TObject* ASender)
{
	FileHandler->SortByName();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortByNameActionUpdate(TObject* ASender)
{
	ViewSortByNameAction->Enabled = FileHandler->CanSortByName();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortByWriteProtectedActionExecute(TObject* ASender)
{
	FileHandler->SortByWriteProtected();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortByWriteProtectedActionUpdate(TObject* ASender)
{
	ViewSortByWriteProtectedAction->Enabled = FileHandler->CanSortByWriteProtected();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortByDiskTypeActionExecute(TObject* ASender)
{
	FileHandler->SortByDiskType();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortByDiskTypeActionUpdate(TObject* ASender)
{
	ViewSortByDiskTypeAction->Enabled = FileHandler->CanSortByDiskType();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortBySizeActionExecute(TObject* ASender)
{
	FileHandler->SortBySize();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewSortBySizeActionUpdate(TObject* ASender)
{
	ViewSortBySizeAction->Enabled = FileHandler->CanSortBySize();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewReverseOrderActionExecute(TObject* ASender)
{
	FileHandler->ReverseOrder();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewReverseOrderActionUpdate(TObject* ASender)
{
	ViewReverseOrderAction->Enabled = FileHandler->CanReverseOrder();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewMoveUpActionExecute(TObject* ASender)
{
	FileHandler->MoveUp();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewMoveUpActionUpdate(TObject* ASender)
{
	ViewMoveUpAction->Enabled = FileHandler->CanMoveUp();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewMoveDownActionExecute(TObject* ASender)
{
	FileHandler->MoveDown();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewMoveDownActionUpdate(TObject* ASender)
{
	ViewMoveDownAction->Enabled = FileHandler->CanMoveDown();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::HelpContentsActionExecute(TObject* ASender)
{
	HelpHandler->ShowContents();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::HelpContentsActionUpdate(TObject* ASender)
{
	HelpContentsAction->Enabled = HelpHandler->CanShowContents();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::HelpAboutActionExecute(TObject* ASender)
{
	HelpHandler->ShowAbout();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::HelpAboutActionUpdate(TObject* ASender)
{
	HelpAboutAction->Enabled = HelpHandler->CanShowAbout();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FormFormAcExecute(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FormFormActionUpdate(TObject* ASender)
{
	AnsiString LCaption = AnsiString().sprintf(
		FormCaptionFormat.c_str(),
		ProgramTitle.c_str(),
		FileHandler->Name.c_str());
	if(LCaption != Caption) {
		Caption = LCaption;
	}
	if(Application->Title != LCaption) {
		Application->Title = LCaption;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FormStatusBar1AcExecute(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FormStatusBar1ActionUpdate(TObject* ASender)
{
	AnsiString LStatus;
	if(ListView1->SelCount > 0) {
		LStatus = AnsiString().sprintf(
			SelectionStatusFormat.c_str(),
			ListView1->SelCount);
	}
	else {
		LStatus = AnsiString().sprintf(
			ItemCountStatusFormat.c_str(),
			ListView1->Items->Count);
	}
	if(LStatus != StatusBar1->SimpleText) {
		StatusBar1->SimpleText = LStatus;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ApplicationIdle(TObject* ASender, bool& ADone)
{
	Application->OnIdle = NULL;
	ADone = true;
	HandleCommandLineArguments();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FormShow(TObject* ASender)
{
	OnShow = NULL;
	LoadSettings();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	FileNewAction->Execute();
	SaveSettings();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ListView1Change(TObject* ASender, TListItem* AItem, TItemChange AChange)
{
	UpdateNotifier->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ListView1ColumnClick(TObject* ASender, TListColumn* AColumn)
{
	switch(AColumn->Index) {
	case 0:
		ViewSortByNameAction->Execute();
		break;
	case 1:
		ViewSortByWriteProtectedAction->Execute();
		break;
	case 2:
		ViewSortByDiskTypeAction->Execute();
		break;
	case 3:
		ViewSortBySizeAction->Execute();
		break;
	default:
		throw EInvalidOperation("The program cannot sort the items.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ListView1Data(TObject* ASender, TListItem* AItem)
{
	if(AItem->Index < 0 || AItem->Index >= FileHandler->Count) {
		return;
	}
	TDiskImage* LDiskImage = FileHandler->Items[AItem->Index];
	AItem->ImageIndex = LDiskImage->WriteProtected == TDiskImage::WriteProtected_Yes ? 1 : 0;
	AItem->Caption = TDiskImageUtilities::DescribeName(LDiskImage->Name);
	AItem->SubItems->Add(TDiskImageUtilities::DescribeWriteProtected(LDiskImage->WriteProtected));
	AItem->SubItems->Add(TDiskImageUtilities::DescribeDiskType(LDiskImage->DiskType));
	AItem->SubItems->Add(TDiskImageUtilities::DescribeSize(LDiskImage->Size));
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAppMainForm::StringToActionCaption(const AnsiString& AString)
{
	return TAnsiStringUtilities::Replace(AString, "&", "&&", true, true);
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ClearShortCut(TMenuItem* AMenuItem)
{
	AMenuItem->ShortCut = 0;
	for(int LI = 0; LI < AMenuItem->Count; ++LI) {
		ClearShortCut(AMenuItem->Items[LI]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::Initialize()
{
	try {
		ClearShortCut(PopupMenu1->Items);
		ClearShortCut(PopupMenu2->Items);
		FFileDropFilesAction = new TFileDropFilesAction(ActionList1);
		FileDropFilesAction->ActionList = ActionList1;
		FileDropFilesAction->OnExecute = FileDropFilesActionExecute;
		FileDropFilesAction->OnUpdate = FileDropFilesActionUpdate;
		FListViewShell = new TListViewShell(ListView1);
		FSystemHandler = new TSystemHandler();
		SystemHandler->OnChange = SystemHandlerChange;
		FFileHandler = new TFileHandler(ListViewShell);
		FileHandler->OnChange = FileHandlerChange;
		FViewHandler = new TViewHandler(ToolBar1, StatusBar1, ListView1);
		ViewHandler->OnChange = ViewHandlerChange;
		FHelpHandler = new THelpHandler();
		HelpHandler->OnChange = HelpHandlerChange;
		FSettingsHandler = new TSettingsHandler();
		SettingsHandler->Handlers->Add(SystemHandler);
		SettingsHandler->Handlers->Add(FileHandler);
		SettingsHandler->Handlers->Add(ViewHandler);
		SettingsHandler->Handlers->Add(HelpHandler);
		SettingsHandler->Handlers->Add(SettingsHandler);
		SettingsHandler->OnChange = SettingsHandlerChange;
		FFileDropWatcher = new TFileDropWatcher(this);
		FileDropWatcher->Enabled = true;
		FileDropWatcher->OnDrop = FileDropWatcherDrop;
		FUpdateNotifier = new TUpdateNotifier();
		UpdateNotifier->OnUpdate = UpdateNotifierUpdate;
		UpdateNotifier->Refresh();
		Application->OnIdle = ApplicationIdle;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::Uninitialize()
{
	delete FUpdateNotifier;
	FUpdateNotifier = NULL;

	delete FFileDropWatcher;
	FFileDropWatcher = NULL;

	delete FSettingsHandler;
	FSettingsHandler = NULL;

	delete FHelpHandler;
	FHelpHandler = NULL;

	delete FViewHandler;
	FViewHandler = NULL;

	delete FSystemHandler;
	FSystemHandler = NULL;

	delete FListViewShell;
	FListViewShell = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::LoadSettings()
{
	SettingsHandler->Load();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::SaveSettings()
{
	SettingsHandler->Save();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::HandleCommandLineArguments()
{
	if(ParamCount() == 0) {
		return;
	}
	try {
		FileDropFilesAction->Files->Clear();
		FileDropFilesAction->Files->Capacity = ParamCount();
		for(int LI = 0; LI < ParamCount(); ++LI) {
			FileDropFilesAction->Files->Add(ParamStr(LI + 1));
		}
		FileDropFilesAction->Execute();
	}
	catch(...) {
		FileDropFilesAction->Files->Clear();
		throw;
	}
	FileDropFilesAction->Files->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::SystemHandlerChange(TObject* ASender)
{
	UpdateNotifier->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileHandlerChange(TObject* ASender)
{
	UpdateNotifier->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::ViewHandlerChange(TObject* ASender)
{
	UpdateNotifier->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::HelpHandlerChange(TObject* ASender)
{
	UpdateNotifier->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::SettingsHandlerChange(TObject* ASender)
{
	UpdateNotifier->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::FileDropWatcherDrop(TObject* ASender, TDroppedFiles* AFiles, bool& AHandled)
{
	AHandled = true;
	::SetForegroundWindow(Handle);
	try {
		FileDropFilesAction->Files->Clear();
		FileDropFilesAction->Files->Capacity = AFiles->Count;
		for(int LI = 0; LI < AFiles->Count; ++LI) {
			FileDropFilesAction->Files->Add(AFiles->Files[LI]);
		}
		AFiles->Finish();
		FileDropFilesAction->Execute();
	}
	catch(...) {
		FileDropFilesAction->Files->Clear();
		throw;
	}
	FileDropFilesAction->Files->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TAppMainForm::UpdateNotifierUpdate(TObject* ASender)
{
	for(int LI = 0; LI < ActionList1->ActionCount; ++LI) {
		ActionList1->Actions[LI]->OnUpdate(ActionList1->Actions[LI]);
	}
}
//---------------------------------------------------------------------------
__fastcall TAppMainForm::TAppMainForm(TComponent* AOwner) :
	TForm(AOwner),
	FFileDropFilesAction(NULL),
	FListViewShell(NULL),
	FFileHandler(NULL),
	FViewHandler(NULL),
	FHelpHandler(NULL),
	FSettingsHandler(NULL),
	FFileDropWatcher(NULL),
	FUpdateNotifier(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TAppMainForm::~TAppMainForm()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TAppMainForm::IsShortCut(TWMKey& AMessage)
{
	if(ListView1->IsEditing()) {
		return false;
	}
	return TForm::IsShortCut(AMessage);
}
//---------------------------------------------------------------------------

