//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2011-09-10 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_CommandPattern.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_commandpattern {
//---------------------------------------------------------------------------
bool __fastcall TCommand::DoExecute()
{
	bool LHandled = false;
	if(OnExecute != NULL) {
		OnExecute(this, LHandled);
	}
	return LHandled;
}
//---------------------------------------------------------------------------
bool __fastcall TCommand::DoUndo()
{
	bool LHandled = false;
	if(OnUndo != NULL) {
		OnUndo(this, LHandled);
	}
	return LHandled;
}
//---------------------------------------------------------------------------
bool __fastcall TCommand::DoRedo()
{
	bool LHandled = false;
	if(OnRedo != NULL) {
		OnRedo(this, LHandled);
	}
	return LHandled;
}
//---------------------------------------------------------------------------
__fastcall TCommand::TCommand() :
	TObject(),
	FOnExecute(NULL),
	FOnUndo(NULL),
	FOnRedo(NULL),
	FResults()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TCommand::~TCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TCommand::Execute()
{
	if(!DoExecute()) {
		ExecuteDefault();
	}
}
//---------------------------------------------------------------------------
void __fastcall TCommand::ExecuteDefault()
{
	// Can be overridden by descendant classes
}
//---------------------------------------------------------------------------
void __fastcall TCommand::Undo()
{
	if(!DoUndo()) {
		UndoDefault();
	}
}
//---------------------------------------------------------------------------
void __fastcall TCommand::UndoDefault()
{
	// Can be overridden by descendant classes
}
//---------------------------------------------------------------------------
void __fastcall TCommand::Redo()
{
	if(!DoRedo()) {
		RedoDefault();
	}
}
//---------------------------------------------------------------------------
void __fastcall TCommand::RedoDefault()
{
	// Can be overridden by descendant classes
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TInvoker::Initialize()
{
	try {
		FQueue = new TObjectList;
		FHistory = new TObjectList;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Uninitialize()
{
	delete FHistory;
	FHistory = NULL;

	delete FQueue;
	FQueue = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::ExpandHistory()
{
	if(MaxHistorySize == 0) {
		return;
	}
	BeginChange();
	try {
		if(MaxHistorySize > 0 && HistorySize == MaxHistorySize) {
			HistorySize = HistorySize / 2;
		}
		History->Expand();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetCommand(int AIndex)
{
	return static_cast<TCommand*>(Queue->Items[AIndex]);
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetNextCommand()
{
	return Commands[0];
}
//---------------------------------------------------------------------------
int __fastcall TInvoker::GetCount()
{
	return Queue->Count;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::SetCount(int ACount)
{
	if(ACount < 0 || ACount > Count) {
		throw ECommandError("The program cannot set the number of the commands.");
	}
	if(ACount == Count) {
		return;
	}
	BeginChange();
	try {
		Queue->Count = ACount;
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetUndoCommand(int AIndex)
{
	return static_cast<TCommand*>(History->Items[HistoryIndex - 1 - AIndex]);
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetNextUndoCommand()
{
	return UndoCommands[0];
}
//---------------------------------------------------------------------------
int __fastcall TInvoker::GetUndoCount()
{
	return HistoryIndex;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::SetUndoCount(int AUndoCount)
{
	if(AUndoCount < 0 || AUndoCount > UndoCount) {
		throw ECommandError("The program cannot set the number of the undo commands.");
	}
	BeginChange();
	try {
		int LToDeleteCount = UndoCount - AUndoCount;
		FHistoryIndex -= LToDeleteCount;
		if(ModifyIndex < LToDeleteCount) {
			FModifyIndex = -1;
		}
		else {
			FModifyIndex -= LToDeleteCount;
		}
		for(int LI = 0; LI < History->Count - LToDeleteCount; ++LI) {
			History->Exchange(LI, LI + LToDeleteCount);
		}
		History->Count = History->Count - LToDeleteCount;
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetRedoCommand(int AIndex)
{
	return static_cast<TCommand*>(History->Items[HistoryIndex + AIndex]);
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetNextRedoCommand()
{
	return RedoCommands[0];
}
//---------------------------------------------------------------------------
int __fastcall TInvoker::GetRedoCount()
{
	return HistorySize - HistoryIndex;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::SetRedoCount(int ARedoCount)
{
	BeginChange();
	try {
		if(ARedoCount < 0 || ARedoCount > RedoCount) {
			throw ECommandError("The program cannot set the number of the redo commands.");
		}
		int LToDeleteCount = RedoCount - ARedoCount;
		if(ModifyIndex > HistorySize - LToDeleteCount) {
			FModifyIndex = -1;
		}
		History->Count = History->Count - LToDeleteCount;
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::SetMaxHistorySize(int AMaxHistorySize)
{
	int LMaxHistorySize = AMaxHistorySize;
	if(LMaxHistorySize < 0) {
		LMaxHistorySize = -1;
	}
	if(LMaxHistorySize == MaxHistorySize) {
		return;
	}
	BeginChange();
	try {
		if(LMaxHistorySize >= 0 && HistorySize > LMaxHistorySize) {
			HistorySize = LMaxHistorySize;
		}
		FMaxHistorySize = LMaxHistorySize;
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
int __fastcall TInvoker::GetHistorySize()
{
	return History->Count;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::SetHistorySize(int AHistorySize)
{
	if(AHistorySize < 0 || AHistorySize > HistorySize) {
		throw ECommandError("The program cannot set the number of the history commands.");
	}
	if(AHistorySize == HistorySize) {
		return;
	}
	BeginChange();
	try {
		if(RedoCount < (HistorySize - AHistorySize)) {
			RedoCount = 0;
			UndoCount = AHistorySize;
		}
		else {
			RedoCount = RedoCount - (HistorySize - AHistorySize);
		}
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::SetModifyIndex(int AModifyIndex)
{
	if(AModifyIndex > HistorySize) {
		throw ECommandError("The program cannot set the modify index.");
	}
	int LModifyIndex = AModifyIndex;
	if(LModifyIndex < 0) {
		LModifyIndex = -1;
	}
	if(LModifyIndex == ModifyIndex) {
		return;
	}
	FModifyIndex = LModifyIndex;
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::FinishExecute()
{
	BeginChange();
	try {
		if(NextCommand->Results.Contains(crClear)) {
			Count = 1;
		}
		if(NextCommand->Results.Contains(crClearUndo)) {
			ClearUndo();
		}
		if(NextCommand->Results.Contains(crClearRedo)) {
			ClearRedo();
		}
		if(NextCommand->Results.Contains(crRemember)) {
			if(NextCommand->Results.Contains(crRetry)) {
				// Do nothing
			}
			else {
				RedoCount = 0;
				ExpandHistory();
				History->Add(NextCommand);
				Queue->Extract(NextCommand);
				++FHistoryIndex;
			}
		}
		else {
			Queue->Delete(0);
		}
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::FinishUndo()
{
	BeginChange();
	try {
		if(NextUndoCommand->Results.Contains(crClear)) {
			Clear();
		}
		if(NextUndoCommand->Results.Contains(crClearUndo)) {
			UndoCount = 1;
		}
		if(NextUndoCommand->Results.Contains(crClearRedo)) {
			ClearRedo();
		}
		if(NextUndoCommand->Results.Contains(crRemember)) {
			if(NextUndoCommand->Results.Contains(crRetry)) {
				// Do nothing
			}
			else {
				--FHistoryIndex;
			}
		}
		else {
			if(ModifyIndex == HistoryIndex - 1) {
				FModifyIndex = -1;
			}
			History->Delete(HistoryIndex - 1);
			--FHistoryIndex;
		}
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::FinishRedo()
{
	BeginChange();
	try {
		if(NextRedoCommand->Results.Contains(crClear)) {
			Clear();
		}
		if(NextRedoCommand->Results.Contains(crClearUndo)) {
			ClearUndo();
		}
		if(NextRedoCommand->Results.Contains(crClearRedo)) {
			RedoCount = 1;
		}
		if(NextRedoCommand->Results.Contains(crRemember)) {
			if(NextRedoCommand->Results.Contains(crRetry)) {
				// Do nothing
			}
			else {
				++FHistoryIndex;
			}
		}
		else {
			if(ModifyIndex == HistoryIndex) {
				FModifyIndex = -1;
			}
			History->Delete(HistoryIndex);
		}
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::DoExecuting()
{
	if(OnExecuting != NULL) {
		OnExecuting(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::DoExecute()
{
	if(OnExecute != NULL) {
		OnExecute(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::DoUndoing()
{
	if(OnUndoing != NULL) {
		OnUndoing(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::DoUndo()
{
	if(OnUndo != NULL) {
		OnUndo(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::DoRedoing()
{
	if(OnRedoing != NULL) {
		OnRedoing(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::DoRedo()
{
	if(OnRedo != NULL) {
		OnRedo(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TInvoker::TInvoker() :
	TObject(),
	FQueue(NULL),
	FHistory(NULL),
	FMaxHistorySize(-1),
	FHistoryIndex(0),
	FModifyIndex(0),
	FChangeCount(0),
	FOnExecuting(NULL),
	FOnExecute(NULL),
	FOnUndoing(NULL),
	FOnUndo(NULL),
	FOnRedoing(NULL),
	FOnRedo(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TInvoker::~TInvoker()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Add(TCommand* ACommand)
{
	if(ACommand == NULL) {
		throw ECommandError("The program cannot add the command.");
	}
	BeginChange();
	try {
		Queue->Add(ACommand);
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Clear()
{
	BeginChange();
	try {
		Queue->Clear();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Execute()
{
	if(!CanExecute()) {
		throw ECommandError("The program cannot execute any commands.");
	}
	BeginChange();
	try {
		NextCommand->Results = TCommandResults() << crClear;
		DoExecuting();
		try {
			NextCommand->Execute();
		}
		catch(...) {
			DoExecute();
			FinishExecute();
			throw;
		}
		DoExecute();
		FinishExecute();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TInvoker::CanExecute()
{
	return Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Undo()
{
	if(!CanUndo()) {
		throw ECommandError("The program cannot undo any commands.");
	}
	BeginChange();
	try {
		NextUndoCommand->Results = TCommandResults() << crClearUndo;
		DoUndoing();
		try {
			NextUndoCommand->Undo();
		}
		catch(...) {
			DoUndo();
			FinishUndo();
			throw;
		}
		DoUndo();
		FinishUndo();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TInvoker::CanUndo()
{
	return UndoCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::ClearUndo()
{
	UndoCount = 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Redo()
{
	if(!CanRedo()) {
		throw ECommandError("The program cannot redo any commands.");
	}
	BeginChange();
	try {
		NextRedoCommand->Results = TCommandResults() << crClearRedo;
		DoRedoing();
		try {
			NextRedoCommand->Redo();
		}
		catch(...) {
			DoRedo();
			FinishRedo();
			throw;
		}
		DoRedo();
		FinishRedo();
	}
	catch(...) {
		EndChange();
		throw;
	}
	EndChange();
}
//---------------------------------------------------------------------------
bool __fastcall TInvoker::CanRedo()
{
	return RedoCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::ClearRedo()
{
	RedoCount = 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::ClearHistory()
{
	HistorySize = 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::ResetModified()
{
	if(ModifyIndex == HistoryIndex) {
		return;
	}
	FModifyIndex = HistoryIndex;
}
//---------------------------------------------------------------------------
bool __fastcall TInvoker::IsModified()
{
	return ModifyIndex != HistoryIndex;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Change()
{
	if(ChangeCount > 0) {
		return;
	}
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::BeginChange()
{
	++FChangeCount;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::EndChange()
{
	if(ChangeCount == 0) {
		throw ECommandError("The program cannot end the change operation.");
	}
	--FChangeCount;
	Change();
}
//---------------------------------------------------------------------------
} // End namespace Moa_commandpattern
//---------------------------------------------------------------------------

