//---------------------------------------------------------------------------
#ifndef Moa_CommandPatternH
#define Moa_CommandPatternH
//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2011-09-10 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_commandpattern {
//---------------------------------------------------------------------------
enum TCommandResult
{
	crClear,
	crClearUndo,
	crClearRedo,
	crRemember,
	crRetry,
};
//---------------------------------------------------------------------------
typedef Set<TCommandResult, crClear, crRetry> TCommandResults;
//---------------------------------------------------------------------------
class ECommandError : public Exception
{
public:
	__fastcall ECommandError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TCommandExecuteEvent)(TObject* ASender, bool& AHandled);
//---------------------------------------------------------------------------
class TCommand : public TObject
{
private:
	TCommandExecuteEvent FOnExecute;
	TCommandExecuteEvent FOnUndo;
	TCommandExecuteEvent FOnRedo;
	TCommandResults FResults;
protected:
	virtual bool __fastcall DoExecute();
	virtual bool __fastcall DoUndo();
	virtual bool __fastcall DoRedo();
public:
	__fastcall TCommand();
	virtual __fastcall ~TCommand();
	virtual void __fastcall Execute();
	virtual void __fastcall ExecuteDefault();
	virtual void __fastcall Undo();
	virtual void __fastcall UndoDefault();
	virtual void __fastcall Redo();
	virtual void __fastcall RedoDefault();
	__property TCommandExecuteEvent OnExecute = {read=FOnExecute, write=FOnExecute};
	__property TCommandExecuteEvent OnUndo = {read=FOnUndo, write=FOnUndo};
	__property TCommandExecuteEvent OnRedo = {read=FOnRedo, write=FOnRedo};
	__property TCommandResults Results = {read=FResults, write=FResults};
};
//---------------------------------------------------------------------------
class TInvoker : public TObject
{
private:
	TObjectList* FQueue;
	TObjectList* FHistory;
	int FMaxHistorySize;
	int FHistoryIndex;
	int FModifyIndex;
	int FChangeCount;
	TNotifyEvent FOnExecuting;
	TNotifyEvent FOnExecute;
	TNotifyEvent FOnUndoing;
	TNotifyEvent FOnUndo;
	TNotifyEvent FOnRedoing;
	TNotifyEvent FOnRedo;
	TNotifyEvent FOnChange;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall ExpandHistory();
	TCommand* __fastcall GetCommand(int AIndex);
	TCommand* __fastcall GetNextCommand();
	int __fastcall GetCount();
	void __fastcall SetCount(int ACount);
	TCommand* __fastcall GetUndoCommand(int AIndex);
	TCommand* __fastcall GetNextUndoCommand();
	int __fastcall GetUndoCount();
	void __fastcall SetUndoCount(int AUndoCount);
	TCommand* __fastcall GetRedoCommand(int AIndex);
	TCommand* __fastcall GetNextRedoCommand();
	int __fastcall GetRedoCount();
	void __fastcall SetRedoCount(int ARedoCount);
	void __fastcall SetMaxHistorySize(int AMaxHistorySize);
	int __fastcall GetHistorySize();
	void __fastcall SetHistorySize(int AHistorySize);
	void __fastcall SetModifyIndex(int AModifyIndex);
	__property TObjectList* Queue = {read=FQueue};
	__property TObjectList* History = {read=FHistory};
	__property int ChangeCount = {read=FChangeCount};
protected:
	virtual void __fastcall FinishExecute();
	virtual void __fastcall FinishUndo();
	virtual void __fastcall FinishRedo();
	virtual void __fastcall DoExecuting();
	virtual void __fastcall DoExecute();
	virtual void __fastcall DoUndoing();
	virtual void __fastcall DoUndo();
	virtual void __fastcall DoRedoing();
	virtual void __fastcall DoRedo();
	virtual void __fastcall DoChange();
public:
	__fastcall TInvoker();
	virtual __fastcall ~TInvoker();
	virtual void __fastcall Add(TCommand* ACommand);
	virtual void __fastcall Clear();
	virtual void __fastcall Execute();
	virtual bool __fastcall CanExecute();
	virtual void __fastcall Undo();
	virtual bool __fastcall CanUndo();
	virtual void __fastcall ClearUndo();
	virtual void __fastcall Redo();
	virtual bool __fastcall CanRedo();
	virtual void __fastcall ClearRedo();
	virtual void __fastcall ClearHistory();
	virtual void __fastcall ResetModified();
	virtual bool __fastcall IsModified();
	virtual void __fastcall Change();
	virtual void __fastcall BeginChange();
	virtual void __fastcall EndChange();
	__property TCommand* Commands[int AIndex] = {read=GetCommand};
	__property TCommand* NextCommand = {read=GetNextCommand};
	__property int Count = {read=GetCount, write=SetCount};
	__property TCommand* UndoCommands[int AIndex] = {read=GetUndoCommand};
	__property TCommand* NextUndoCommand = {read=GetNextUndoCommand};
	__property int UndoCount = {read=GetUndoCount, write=SetUndoCount};
	__property TCommand* RedoCommands[int AIndex] = {read=GetRedoCommand};
	__property TCommand* NextRedoCommand = {read=GetNextRedoCommand};
	__property int RedoCount = {read=GetRedoCount, write=SetRedoCount};
	__property int MaxHistorySize = {read=FMaxHistorySize, write=SetMaxHistorySize};
	__property int HistorySize = {read=GetHistorySize, write=SetHistorySize};
	__property int HistoryIndex = {read=FHistoryIndex};
	__property int ModifyIndex = {read=FModifyIndex, write=SetModifyIndex};
	__property TNotifyEvent OnExecuting = {read=FOnExecuting, write=FOnExecuting};
	__property TNotifyEvent OnExecute = {read=FOnExecute, write=FOnExecute};
	__property TNotifyEvent OnUndoing = {read=FOnUndoing, write=FOnUndoing};
	__property TNotifyEvent OnUndo = {read=FOnUndo, write=FOnUndo};
	__property TNotifyEvent OnRedoing = {read=FOnRedoing, write=FOnRedoing};
	__property TNotifyEvent OnRedo = {read=FOnRedo, write=FOnRedo};
	__property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------
} // End namespace Moa_commandpattern
using namespace Moa_commandpattern;
//---------------------------------------------------------------------------
#endif

