//---------------------------------------------------------------------------
#ifndef Moa_FilesH
#define Moa_FilesH
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2011-08-27 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
namespace Moa_files {
//---------------------------------------------------------------------------
class EFileError : public Exception
{
public:
	__fastcall EFileError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TPathUtilities : public TObject
{
private:
protected:
public:
	static const char* DeviceNames[];
	static const int DeviceNameCount;
	static const char* DotName;
	static const char* DotDotName;
	static const char* Separators;
	static const char* WildcardCharacters;
	static const char DriveSeparator;
	static const char DirectorySeparator;
	static const char ExtensionSeparator;
	static const char AnyWildcardCharacter;
	static const char OneWildcardCharacter;
	static const bool PathCharacterTable[];
	static const int PathCharacterTableSize;
	static const bool NameCharacterTable[];
	static const int NameCharacterTableSize;
	static const bool SeparatorTable[];
	static const int SeparatorTableSize;
	static const bool WildcardCharacterTable[];
	static const int WildcardCharacterTableSize;
	static AnsiString __fastcall BeginWithSeparator(const AnsiString& APath, char ASeparator);
	static bool __fastcall BeginsWithSeparator(const AnsiString& APath, char ASeparator);
	static AnsiString __fastcall EndWithSeparator(const AnsiString& APath, char ASeparator);
	static bool __fastcall EndsWithSeparator(const AnsiString& APath, char ASeparator);
	static AnsiString __fastcall ReplaceInvalidPathCharacters(const AnsiString& APath, char ANewCharacter);
	static AnsiString __fastcall ReplaceInvalidNameCharacters(const AnsiString& AName, char ANewCharacter);
	static AnsiString __fastcall RemoveInvalidPathCharacters(const AnsiString& APath);
	static AnsiString __fastcall RemoveInvalidNameCharacters(const AnsiString& AName);
	static int __fastcall FindSeparator(const AnsiString& APath);
	static int __fastcall FindInvalidPathCharacter(const AnsiString& APath);
	static int __fastcall FindInvalidNameCharacter(const AnsiString& AName);
	static int __fastcall FindWildcardCharacter(const AnsiString& APath);
	static bool __fastcall IsAbsolutePath(const AnsiString& APath);
	static bool __fastcall IsRelativePath(const AnsiString& APath);
	static bool __fastcall IsDeviceName(const AnsiString& AName);
	static bool __fastcall IsDotName(const AnsiString& AName);
	static bool __fastcall IsDotDotName(const AnsiString& AName);
	static bool __fastcall IsPathCharacter(char ACharacter);
	static bool __fastcall IsNameCharacter(char ACharacter);
	static bool __fastcall IsSeparator(char ACharacter);
	static bool __fastcall IsWildcardCharacter(char ACharacter);
};
//---------------------------------------------------------------------------
class TFileUtilities : public TObject
{
private:
protected:
public:
	static AnsiString __fastcall GetLongPath(const AnsiString& APath); 
	static bool __fastcall TryGetLongPath(const AnsiString& APath, AnsiString& ALongPath); 
	static AnsiString __fastcall GetShortPath(const AnsiString& APath); 
	static bool __fastcall TryGetShortPath(const AnsiString& APath, AnsiString& AShortPath); 
};
//---------------------------------------------------------------------------
class TDirectoryEntryEnumerator : public TObject
{
private:
	HANDLE FHandle;
	AnsiString FMask;
	WIN32_FIND_DATA FCurrent;
	void __fastcall FindFirst();
	void __fastcall FindNext();
	void __fastcall Finish();
	bool __fastcall IsStarted();
	bool __fastcall IsFinished();
	__property HANDLE Handle = {read=FHandle};
	__property AnsiString Mask = {read=FMask};
protected:
public:
	__fastcall TDirectoryEntryEnumerator(const AnsiString& AMask);
	virtual __fastcall ~TDirectoryEntryEnumerator();
	virtual bool __fastcall MoveNext();
	virtual void __fastcall Reset();
	__property WIN32_FIND_DATA Current = {read=FCurrent};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class TPathList : public TStringList
{
private:
	__property bool CaseSensitive = {};
protected:
	virtual int __fastcall CompareStrings(const AnsiString AString1, const AnsiString AString2);
public:
	__fastcall TPathList();
	virtual __fastcall ~TPathList();
};
//---------------------------------------------------------------------------
} // End namespace Moa_files
using namespace Moa_files;
//---------------------------------------------------------------------------
#endif

