//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2011-08-26 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_Lists.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_lists {
//---------------------------------------------------------------------------
int __fastcall TListOperator::CompareSortRecord(void* AItem1, void* AItem2)
{
	TSortRecord* LItem1 = static_cast<TSortRecord*>(AItem1);
	TSortRecord* LItem2 = static_cast<TSortRecord*>(AItem2);
	int LResult = LItem1->Compare(LItem1->List, LItem1->OldIndex, LItem2->OldIndex, LItem1->Data);
	if(LResult == 0) {
		LResult = LItem1->OldIndex - LItem2->OldIndex;
	}
	return LResult;
}
//---------------------------------------------------------------------------
void __fastcall TListOperator::Sort(TList* AList, TListOperatorSortCompare ACompare, void* AData)
{
	Reorder(AList, SimulateSort(AList, ACompare, AData));
}
//---------------------------------------------------------------------------
TListOperatorReorderRecords __fastcall TListOperator::SimulateSort(TList* AList, TListOperatorSortCompare ACompare, void* AData)
{
	if(AList->Count == 0) {
		return TListOperatorReorderRecords();
	}
	DynamicArray<TSortRecord> LSortRecords;
	{
		LSortRecords.Length = AList->Count;
		for(int LI = 0; LI < LSortRecords.Length; ++LI) {
			LSortRecords[LI].List = AList;
			LSortRecords[LI].OldIndex = LI;
			LSortRecords[LI].Compare = ACompare;
			LSortRecords[LI].Data = AData;
		}
		TAutoPointer<TList> LSortList = new TList();
		LSortList->Count = LSortRecords.Length;
		for(int LI = 0; LI < LSortList->Count; ++LI) {
			LSortList->Items[LI] = &LSortRecords[LI];
		}
		LSortList->Sort(CompareSortRecord);
		for(int LI = 0; LI < LSortList->Count; ++LI) {
			static_cast<TSortRecord*>(LSortList->Items[LI])->NewIndex = LI;
		}
	}
	TListOperatorReorderRecords LReorderRecords;
	{
		int LChangeCount = 0;
		for(int LI = 0; LI < LSortRecords.Length; ++LI) {
			if(LSortRecords[LI].NewIndex == LSortRecords[LI].OldIndex) {
				continue;
			}
			++LChangeCount;
		}
		if(LChangeCount == 0) {
			return TListOperatorReorderRecords();
		}
		LReorderRecords.Length = LChangeCount;
	}
	{
		int LChangeCount = 0;
		for(int LI = 0; LI < LSortRecords.Length; ++LI) {
			TSortRecord& LSortRecord1 = LSortRecords[LI];
			while(LSortRecord1.NewIndex != LSortRecord1.OldIndex) {
				LReorderRecords[LChangeCount].Index1 = LSortRecord1.OldIndex;
				LReorderRecords[LChangeCount].Index2 = LSortRecord1.NewIndex;
				++LChangeCount;
				if(LChangeCount == LReorderRecords.Length) {
					break;
				}
				TSortRecord& LSortRecord2 = LSortRecords[LSortRecord1.NewIndex];
				LSortRecord1.NewIndex = LSortRecord2.NewIndex;
				LSortRecord2.NewIndex = LSortRecord2.OldIndex;
			}
		}
		LReorderRecords.Length = LChangeCount;
	}
	return LReorderRecords;
}
//---------------------------------------------------------------------------
void __fastcall TListOperator::Reverse(TList* AList)
{
	Reorder(AList, SimulateReverse(AList));
}
//---------------------------------------------------------------------------
TListOperatorReorderRecords __fastcall TListOperator::SimulateReverse(TList* AList)
{
	return AbstractReverse(AList->Count);
}
//---------------------------------------------------------------------------
TListOperatorReorderRecords __fastcall TListOperator::AbstractReverse(int ACount)
{
	TListOperatorReorderRecords LReorderRecords;
	LReorderRecords.Length = ACount / 2;
	for(int LI = 0; LI < ACount / 2; ++LI) {
		LReorderRecords[LI].Index1 = LI;
		LReorderRecords[LI].Index2 = ACount - 1 - LI;
	}
	return LReorderRecords;
}
//---------------------------------------------------------------------------
void __fastcall TListOperator::Slide(TList* AList, const TIntegerDynArray& AIndexes, int ADistance)
{
	Reorder(AList, SimulateSlide(AList, AIndexes, ADistance));
}
//---------------------------------------------------------------------------
TListOperatorReorderRecords __fastcall TListOperator::SimulateSlide(TList* AList, const TIntegerDynArray& AIndexes, int ADistance)
{
	return AbstractSlide(AList->Count, AIndexes, ADistance);
}
//---------------------------------------------------------------------------
TListOperatorReorderRecords __fastcall TListOperator::AbstractSlide(int ACount, const TIntegerDynArray& AIndexes, int ADistance)
{
	if(ACount == 0 || ADistance == 0) {
		return TListOperatorReorderRecords();
	}
	DynamicArray<TSlideRecord> LSlideRecords;
	{
		LSlideRecords.Length = ACount;
		for(int LI = 0; LI < LSlideRecords.Length; ++LI) {
			LSlideRecords[LI].OldIndex = LI;
			LSlideRecords[LI].Target = false;
		}
		for(int LI = 0; LI < AIndexes.Length; ++LI) {
			LSlideRecords[AIndexes[LI]].Target = true;
		}
		TAutoPointer<TList> LSlideList = new TList();
		LSlideList->Count = LSlideRecords.Length;
		for(int LI = 0; LI < LSlideList->Count; ++LI) {
			LSlideList->Items[LI] = &LSlideRecords[LI];
		}
		{
			int LSourceIndex;
			int LSourceStep;
			int LMoveCount;
			int LMoveStep;
			if(ADistance < 0) {
				LSourceIndex = -1;
				LSourceStep = 1;
				LMoveCount = -ADistance;
				LMoveStep = -1;
			}
			else {
				LSourceIndex = LSlideList->Count;
				LSourceStep = -1;
				LMoveCount = ADistance;
				LMoveStep = 1;
			}
			for(int LI = 0; LI < ACount; ++LI) {
				LSourceIndex += LSourceStep;
				if(LI == 0 || !static_cast<TSlideRecord*>(LSlideList->Items[LSourceIndex])->Target) {
					continue;
				}
				int LDestinationIndex = LSourceIndex;
				for(int LJ = 0; LJ < LMoveCount; ++LJ) {
					if(LDestinationIndex + LMoveStep < 0 ||
						LDestinationIndex + LMoveStep >= LSlideList->Count ||
						static_cast<TSlideRecord*>(LSlideList->Items[LDestinationIndex + LMoveStep])->Target) {
						break;
					}
					LDestinationIndex += LMoveStep;
				}
				LSlideList->Move(LSourceIndex, LDestinationIndex);
			}
		}
		for(int LI = 0; LI < LSlideList->Count; ++LI) {
			static_cast<TSlideRecord*>(LSlideList->Items[LI])->NewIndex = LI;
		}
	}
	TListOperatorReorderRecords LReorderRecords;
	{
		int LChangeCount = 0;
		for(int LI = 0; LI < LSlideRecords.Length; ++LI) {
			if(LSlideRecords[LI].NewIndex == LSlideRecords[LI].OldIndex) {
				continue;
			}
			++LChangeCount;
		}
		if(LChangeCount == 0) {
			return TListOperatorReorderRecords();
		}
		LReorderRecords.Length = LChangeCount;
	}
	{
		int LChangeCount = 0;
		for(int LI = 0; LI < LSlideRecords.Length; ++LI) {
			TSlideRecord& LSlideRecord1 = LSlideRecords[LI];
			while(LSlideRecord1.NewIndex != LSlideRecord1.OldIndex) {
				LReorderRecords[LChangeCount].Index1 = LSlideRecords[LI].OldIndex;
				LReorderRecords[LChangeCount].Index2 = LSlideRecords[LI].NewIndex;
				++LChangeCount;
				if(LChangeCount == LReorderRecords.Length) {
					break;
				}
				TSlideRecord& LSlideRecord2 = LSlideRecords[LSlideRecord1.NewIndex];
				LSlideRecord1.NewIndex = LSlideRecord2.NewIndex;
				LSlideRecord2.NewIndex = LSlideRecord2.OldIndex;
			}
		}
		LReorderRecords.Length = LChangeCount;
	}
	return LReorderRecords;
}
//---------------------------------------------------------------------------
void __fastcall TListOperator::Reorder(TList* AList, const TListOperatorReorderRecords& ARecords)
{
	for(int LI = 0; LI < ARecords.Length; ++LI) {
		AList->Exchange(ARecords[LI].Index1, ARecords[LI].Index2);
	}
}
//---------------------------------------------------------------------------
} // End namespace Moa_lists
//---------------------------------------------------------------------------

